var Memory = function(id, rows, cols, suiteName) {
	var r, c;
	var row, cell, board;
	var button, form;
	var container = document.getElementById(id);
		
	this.score = null;
	this.scoreDisplay = null;
	this.cards = [];
	this.suiteName = suiteName;
	this.faceUpCard = [null, null];
	this.timeoutID = null;
	
	board = document.createElement("table");	
	container.appendChild(board);
	
	for (r = 0; r < rows; r++) {
		row = document.createElement("tr");
		board.appendChild(row);
		for (c = 0; c < cols; c++) {
			cell = document.createElement("td");
			row.appendChild(cell);
			this.cards.push(new Card(cell));
		}
	}
	
	form = document.createElement("form");
	container.appendChild(form);
		
	button = document.createElement("input");
	button.type = "button";
	button.value= "Start New Game";
	button.onclick = startNewGame(this);
	form.appendChild(button);
		
	this.scoreDisplay = document.createElement("label");
	form.appendChild(this.scoreDisplay);		
	
	startNewGame(this)();
};
	
Memory.prototype.updateScore = function(score) {
	if (score != 0) {
		this.score += score;
	}
	else {
		this.score = 0;
	}
	this.scoreDisplay.innerHTML = "Score: " + this.score;
};
	
Memory.prototype.processFaceUpCards = function() {
	if (this.faceUpCard[0].getIndex() == this.faceUpCard[1].getIndex()) {
		this.faceUpCard[0].remove();
		this.faceUpCard[1].remove();
		this.updateScore(2);
	}
	else {
		this.faceUpCard[0].turn(false); //Turn face down
		this.faceUpCard[1].turn(false); //Turn face down
		this.updateScore(-1);
	}
	this.faceUpCard[0] = null;
	this.faceUpCard[1] = null;
};

Memory.prototype.setTimeout = function(timeout) {
	this.timeoutID = setTimeout(handleTimeout(this), timeout);
};

Memory.prototype.clearTimeout = function() {
	clearTimeout(this.timeoutID);
};

Memory.prototype.geNumberOfCards = function() {
	return this.cards.length;
};

Memory.prototype.getCard = function(i) {
	return this.cards[i];
};

Memory.prototype.getSuiteName = function(i) {
	return this.suiteName;
};

Memory.prototype.getFaceUpCard = function(num) {
	return this.faceUpCard[num];
};	
	
Memory.prototype.setFaceUpCard = function(num, card) {
	this.faceUpCard[num] = card;
};	

var Card = function(cell) {
	this.cardIndex = null;
	this.imageFileName = null;
	this.faceUp = null;
	this.image = document.createElement("img");
	
	cell.appendChild(this.image);
};

Card.prototype.init = function(index, game) {
	this.cardIndex = index;
	this.imageFileName = game.getSuiteName() + index + ".png"
	this.image.className = "visible";
	this.image.onclick = handleCardClick(game, this);
	this.turn(false); //Turn face down
};
	
Card.prototype.remove = function() {
	this.cardIndex = -1;
	this.imageFileName = null;
	this.image.onclick = null;
	this.image.className = "hidden";
};

Card.prototype.turn = function(faceUp) {
	this.faceUp = faceUp;
	if (faceUp == false) {
		this.image.src = "back_side.png"; 
	}
	else {
		this.image.src = this.imageFileName; 
	}
};

Card.prototype.isFaceUp = function() {
	return this.faceUp;
};

Card.prototype.getIndex = function() {
	return this.cardIndex;
};
		
var handleCardClick = function(game, card) {
	return function() {
		if (card.isFaceUp() == true) {
			return;
		}
		if (game.getFaceUpCard(0) !== null) {
			if (game.getFaceUpCard(1) !== null) {
				game.processFaceUpCards();
				game.clearTimeout();
				game.setFaceUpCard(0, card);
			}
			else {
				game.setFaceUpCard(1, card);
				game.setTimeout(2000);
			}
		}
		else {
			game.setFaceUpCard(0, card);;
		}
		card.turn(true); //Turn over
	};
};
	
var handleTimeout = function(game) {
	return function() {
		game.processFaceUpCards();
	};
};

var startNewGame = function(game) {
	return function() {
		var i, j, k, tmp, cardIndexes = [];
		var images;
		for (i = 0; i < game.geNumberOfCards() / 2; i++) {
			cardIndexes.push(i + 1);
			cardIndexes.push(i + 1);
		}
		for (i = 0; i < 1000; i++) {
			j = Math.floor(Math.random() * cardIndexes.length);
			k = Math.floor(Math.random() * cardIndexes.length);
			tmp = cardIndexes[j];
			cardIndexes[j] = cardIndexes[k];
			cardIndexes[k] = tmp;
		}
		for (i = 0; i < game.geNumberOfCards(); i++) {
			game.getCard(i).init(cardIndexes[i], game);
		}
		game.updateScore(0);
	};
};
	
window.onload = function() {
	new Memory("memory1", 6, 8, "drink");
};
